using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AddinExpress.OL;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.IO;
using System.Xml.Serialization;

namespace adx_ol_cs_AttachmentSerialization
{
    public partial class frmProductInfo : AddinExpress.OL.ADXOlForm
    {
        public frmProductInfo()
        {
            InitializeComponent();
        }

        private void frmProductInfo_ADXBeforeFormShow()
        {
            Outlook.Inspector currInsp = null;
            Outlook.MailItem mail = null;
            Outlook.Attachments attachments = null;
            object item = null;
            Boolean success = false;

            try
            {
                currInsp = InspectorObj as Outlook.Inspector;
                if (currInsp != null)
                {
                    item = currInsp.CurrentItem;
                    if (item is Outlook.MailItem)
                    {
                        mail = item as Outlook.MailItem;
                        attachments = mail.Attachments;
                        if (!mail.Sent)
                        {
                            this.Visible = false;
                            return;
                        }

                        for (int i = 1; i <= attachments.Count; i++)
                        {
                            string TempFileTmp = Path.GetTempFileName();
                            string TempFileXml = Path.ChangeExtension(TempFileTmp, "xml");
                            if (File.Exists(TempFileTmp))
                            {
                                File.Delete(TempFileTmp);
                            }

                            Outlook.Attachment attachment = attachments[i];
                            attachment.SaveAsFile(TempFileXml);
                            if (attachment != null) Marshal.ReleaseComObject(attachment);

                            try
                            {
                                Product product;
                                FileStream fileStream = new FileStream(TempFileXml, FileMode.Open);
                                try
                                {
                                    XmlSerializer newSerializer = new XmlSerializer(typeof(Product));
                                    product = (Product)newSerializer.Deserialize(fileStream);
                                    txtID.Text = product.ID;
                                    txtName.Text = product.Name;
                                    txtDiscontinued.Text = product.Discontinued.ToString();
                                    txtCount.Text = product.Count.ToString();
                                    txtDate.Text = product.Date.ToString();
                                    success = true;
                                }
                                finally
                                {
                                    fileStream.Close();
                                }
                            }
                            catch
                            {
                                // skip exception (serialization)
                            }
                            File.Delete(TempFileXml);
                            if (success) break;
                        }

                        if (!success)
                        {
                            this.Visible = false;
                            return;
                        }
                    }
                }
            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
                if (attachments != null) Marshal.ReleaseComObject(attachments);
            }
        }
    }
}
